define(["require", "exports", "../../configs/monsterConfiguration", "../../configs/monsterEmitterConfiguration", "../../models/monsterModel", "../../utils/intervalTimer", "../../utils/mathUtil", "../../configs/difficultyConfiguration", "../../utils/logger"], function (require, exports, monsterConfiguration_1, monsterEmitterConfiguration_1, monsterModel_1, intervalTimer_1, mathUtil_1, difficultyConfiguration_1, logger_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MonsterEmitter {
        constructor(model, playerLogic, configUtil) {
            this._monsterUid = 0;
            this._monsterToSpawn = 0;
            this._spawnMonsterMaxNum = 1;
            this._spawnMonsterLeftNum = 1;
            this._spawnMonsterLevel = 0;
            this._spawnBoss = false;
            this._isBossOnly = false;
            this._model = model;
            this.playerLogic = playerLogic;
            this.config = configUtil.getConfig(monsterEmitterConfiguration_1.MonsterEmitterConfiguration);
            this.dconfig = configUtil.getConfig(difficultyConfiguration_1.DifficultyConfiguration);
            this.mconfig = configUtil.getConfig(monsterConfiguration_1.MonsterConfiguration);
            this.rushTimer = new intervalTimer_1.IntervalTimer(1000);
        }
        get model() {
            return this._model;
        }
        start() {
            this._aliveMonsters = [];
            this.updatePool();
        }
        tryEmit() {
            if (this._monsterToSpawn < 1) {
                let aliveMonsters = [];
                for (let monster of this._aliveMonsters) {
                    if (monster.alive) {
                        aliveMonsters.push(monster);
                    }
                }
                this._aliveMonsters = aliveMonsters;
                if (aliveMonsters.length == 0 || this.rushTimer.checkAndReset()) {
                    if (this._spawnMonsterLeftNum == 0) {
                        if (!this._isBossOnly) {
                            this._spawnBoss = true;
                            this._monsterToSpawn = 1;
                        }
                        this._spawnMonsterLeftNum = -1;
                    }
                    else if (this._spawnMonsterLeftNum > 0) {
                        let num = mathUtil_1.MathUtil.getRandomInt(this._spawnMonsterMaxNum / 8) + 1;
                        if (num > this._spawnMonsterLeftNum) {
                            num = this._spawnMonsterLeftNum;
                        }
                        this._spawnMonsterLeftNum -= num;
                        this._monsterToSpawn = num;
                    }
                }
            }
            if (this._monsterToSpawn > 0) {
                if (this._isBossOnly) {
                    this._spawnBoss = true;
                }
                let info = {
                    emitterModel: this._model,
                    monsterPool: this._spawnBoss ? this._bossPool : this._monsterPool,
                    monsterNum: 1,
                    monsterLevel: this._spawnMonsterLevel,
                    isBoss: this._spawnBoss
                };
                let ret = this.spawnMonsters(info);
                this._aliveMonsters = this._aliveMonsters.concat(ret);
                this._monsterToSpawn -= 1;
                this._spawnBoss = false;
                return ret;
            }
            return null;
        }
        onMonsterDestroyed(model, killed, lastMonster) {
            const em = this._model;
            if (killed) {
                if (model.isBoss && !this._isBossOnly) {
                    em.percentage += 0.1;
                }
                else {
                    em.percentage += 1 / this._spawnMonsterMaxNum;
                }
            }
            if (this._spawnMonsterLeftNum < 0 && lastMonster) {
                const logParams = { level: em.level, wave: em.configId, percentage: em.percentage };
                if (em.percentage > 0.89) {
                    logger_1.Logger.p("wave_completed", logParams);
                    if (this.config.isMaxConfigId(em.level, em.configId)) {
                        em.configId = 1;
                        if (!this.config.isMaxLevel(em.level)) {
                            em.level += 1;
                        }
                        this.playerLogic.handleEvent("difficulty_finished", em);
                    }
                    else {
                        em.configId += 1;
                    }
                }
                else {
                    logger_1.Logger.p("wave_not_completed", logParams);
                }
                this.updatePool();
            }
        }
        updatePool() {
            let configEntry = this.config.getConfig(this._model.configId);
            this._monsterPool = configEntry.monsterPool;
            this._bossPool = configEntry.bossPool;
            const difficulty = this.dconfig.get(this._model.level);
            this._spawnMonsterMaxNum = configEntry.exp * difficulty.numMultiplier;
            this._spawnMonsterLevel = difficulty.monsterLevel;
            this._spawnMonsterLeftNum = this._spawnMonsterMaxNum;
            this._model.percentage = 0;
            this._isBossOnly = this._monsterPool.length == 0;
        }
        spawnMonsters(info) {
            const randIndex = info.emitterModel.getRand() % info.monsterPool.length;
            const configId = info.monsterPool[randIndex];
            const ret = [];
            for (let i = 0; i < info.monsterNum; i++) {
                this._monsterUid += 1;
                const monster = new monsterModel_1.MonsterModel(this._monsterUid.toString());
                const config = this.mconfig.get(configId);
                monster.level = info.monsterLevel;
                monster.moveSpeed = config.moveSpeed;
                monster.configId = configId;
                monster.baseGold = config.gold;
                monster.baseMagicEssence = config.magicEssence;
                monster.baseMaxHp = config.hp;
                monster.hp = monster.maxHp;
                monster.defense = config.def;
                monster.defenseType = config.defType;
                monster.isBoss = info.isBoss;
                ret.push(monster);
            }
            return ret;
        }
    }
    exports.MonsterEmitter = MonsterEmitter;
});
//# sourceMappingURL=monsterEmitter.js.map