define(["require", "exports", "../../utils/eventbus", "../../configs/emotionConfiguration", "../../configs/spritesheetConfiguration", "../../logics/playerLogic"], function (require, exports, eventbus_1, emotionConfiguration_1, spritesheetConfiguration_1, playerLogic_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class EmotionBubble extends Phaser.Sprite {
        get context() {
            return this._context;
        }
        set context(value) {
            this._context = value;
        }
    }
    class BaseCharacter extends Phaser.Sprite {
        constructor(game, x, y, spriteSheetKey) {
            super(game, x, y, spriteSheetKey);
            this._eventBus = new eventbus_1.Eventbus();
            this._initialized = false;
            this._bubbles = [];
            this._buffs = [];
            this._direction = "up";
        }
        get buffs() {
            return this._buffs;
        }
        get direction() {
            return this._direction;
        }
        get field() {
            return this._field;
        }
        set field(value) {
            this._field = value;
        }
        get serviceProvider() {
            return this._field.getCurrentScene().serviceProvider;
        }
        get playerLogic() {
            return this._field.getCurrentScene().getLogic(playerLogic_1.PlayerLogic);
        }
        notifyPropertyChangeS(name, value) {
            this._eventBus.emit(name, value);
        }
        onPropertyChangedS(evt, caller, listener) {
            return this._eventBus.on(evt, (event) => {
                listener.call(caller, event);
            });
        }
        updateFrame() {
            if (!this._initialized) {
                this._initialized = true;
                this.initialize();
            }
            const now = Date.now();
            let temp = new Array();
            for (let buff of this._buffs) {
                if (buff.isFinished) {
                    continue;
                }
                const diff = now - buff.start;
                if (diff < buff.duration) {
                    temp.push(buff);
                }
                else {
                    buff.isTimeout = true;
                }
            }
            if (temp.length != this._buffs.length) {
                this._buffs = temp;
                this.updateStatus();
            }
            let remaining = new Array();
            for (let bubble of this._bubbles) {
                if (bubble.context == null) {
                    continue;
                }
                if (bubble.context.isFinished) {
                    this.removeBubble(bubble);
                }
                else {
                    remaining.push(bubble);
                }
            }
            if (remaining.length != this._bubbles.length) {
                this._bubbles = remaining;
                this.resetBubblePosition();
            }
        }
        initialize() {
            this.initBubbles();
        }
        removeBubble(target) {
            let bubble;
            if (typeof target === "string") {
                for (let b of this._bubbles) {
                    if (b.key == target) {
                        bubble = b;
                        break;
                    }
                }
            }
            else {
                bubble = target;
            }
            if (bubble != null) {
                bubble.pendingDestroy = true;
            }
        }
        addBubble(emotionAlias, context) {
            const scene = this.field.getCurrentScene();
            const config = scene.configUtil.getConfig(emotionConfiguration_1.EmotionConfiguration).getConfig(emotionAlias);
            const spritesheet = scene.configUtil.getConfig(spritesheetConfiguration_1.SpritesheetConfiguration).getConfig(config.pathId);
            const key = "emotion_" + emotionAlias;
            for (let bubble of this._bubbles) {
                if (bubble.key == key) {
                    return;
                }
            }
            scene.checkAndLoadSpriteSheet(key, spritesheet.path, spritesheet.dimension[0], spritesheet.dimension[1], spritesheet.dimension[2], spritesheet.dimension[3], () => {
                if (this.game) {
                    let bubble = new EmotionBubble(this.game, 0, 0, key);
                    bubble.context = context;
                    bubble.anchor.set(0.5, 1);
                    bubble.animations.add("ani", config.animation, 4, true);
                    bubble.play("ani");
                    bubble.scale.set(1.5 / this.scale.x);
                    this._bubbles.push(bubble);
                    this.addChild(bubble);
                    this.resetBubblePosition();
                }
            });
        }
        resetBubblePosition() {
            if (this._bubbles.length < 1) {
                return;
            }
            const bw = this._bubbles[0].width;
            const bh = this._bubbles[0].height;
            let startX = 0 - this.width / this.scale.x / 2 - bw / 2 * (this._bubbles.length - 1);
            for (let bubble of this._bubbles) {
                bubble.position.set(startX, 0 - this.height / 2);
                startX += bw;
            }
        }
        addBuff(buff, showBubble) {
            for (let b of this._buffs) {
                if (b.compareTo(buff) == 0) {
                    b.start = Date.now();
                    if (b.overlap < b.maxOverlap) {
                        b.overlap++;
                        this.updateStatus();
                    }
                    return;
                }
            }
            buff.start = Date.now();
            this._buffs.push(buff);
            this.updateStatus();
            if (showBubble) {
                if (buff.positive == 1) {
                    this.addBubble("statUp", buff);
                }
                else if (buff.positive == -1) {
                    this.addBubble("statDown", buff);
                }
            }
        }
        checkBuffAfterEvent(event) {
            for (let b of this._buffs) {
                if (b.removeEvent == event) {
                    b.times -= 1;
                }
            }
        }
        fight(speed = 1, times = 1) {
            let fightAni = this.animations.getAnimation("fight");
            fightAni.play(10 * speed, true);
            if (fightAni.onLoop.getNumListeners() > 0) {
                fightAni.onLoop.removeAll();
            }
            fightAni.onLoop.add(() => {
                if (fightAni.loopCount >= times) {
                    fightAni.stop();
                    this.standby();
                }
            });
        }
        standby() {
            this.animations.play("standby");
        }
        walk(dir = "right") {
            this.animations.play("walk_" + dir);
            this._direction = dir;
        }
    }
    exports.BaseCharacter = BaseCharacter;
});
//# sourceMappingURL=baseCharacter.js.map