define(["require", "exports", "./util"], function (require, exports, util_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const _ptHelper = new Phaser.Point();
    const defaultOptions = {
        from: 0,
        to: 200,
        direction: 'y',
        momentum: false,
        snapping: false,
        bouncing: false,
        deceleration: 0.5,
        overflow: 20,
        snapStep: 10,
        emitMoving: false,
        duration: 2,
        speedLimit: 3,
        flickTimeThreshold: 100,
        offsetThreshold: 30,
        acceleration: 0.5,
        accelerationT: 250,
        maxAcceleration: 4,
        time: {},
        multiplier: 1,
        swipeEnabled: false,
        swipeThreshold: 5,
        swipeTimeThreshold: 250,
        minDuration: 0.5,
        addListeners: true,
        infinite: false
    };
    class Scroller {
        get events() {
            return this._events;
        }
        get o() {
            return this._o;
        }
        set current(value) {
            this._current = value;
        }
        set old(value) {
            this._old = value;
        }
        set down(value) {
            this._down = value;
        }
        constructor(game, clickObject, maskLimits = {}, options = {}) {
            this.game = game;
            this.clickObject = clickObject;
            this.maskLimits = maskLimits;
            this._o = this._options = Object.assign({}, defaultOptions, options);
            this._updateMinMax();
            this.dispatchValues = { step: 0, total: 0, percent: 0 };
            this.addListeners();
            this.clickables = [];
            this.isDown = false;
            this.isScrolling = false;
            this.scrollObject = {};
            this.init();
            this.tweenScroll = this.game.add.tween(this.scrollObject).to({}, 0, Phaser.Easing.Quartic.Out);
            this.tweenScroll.onUpdateCallback(this.handleUpdate, this);
            this.tweenScroll.onComplete.add(this.handleComplete, this);
        }
        destroy() {
            this.tweenScroll.stop();
            this.removeListeners();
            this.clickObject.destroy();
            this.clickables = null;
            this._options = this._o = null;
            this.maskLimits = null;
            this.enabled = false;
            this.game = null;
            this.dispatchValues = null;
            this.isDown = null;
            this.target = null;
            this.destroyed = true;
        }
        addListeners() {
            this._events = {
                onUpdate: new Phaser.Signal(),
                onInputUp: new Phaser.Signal(),
                onInputDown: new Phaser.Signal(),
                onInputMove: new Phaser.Signal(),
                onComplete: new Phaser.Signal(),
                onSwipe: new Phaser.Signal()
            };
            if (this._o.addListeners) {
                this.clickObject.inputEnabled = true;
                this.clickObject.events.onInputDown.add(this.handleDown, this);
                this.clickObject.events.onInputUp.add(this.handleUp, this);
            }
        }
        removeListeners() {
            if (this._o.addListeners) {
                this.clickObject.events.onInputDown.remove(this.handleDown, this);
                this.clickObject.events.onInputUp.remove(this.handleUp, this);
            }
            for (let property in this._events) {
                if (this._events.hasOwnProperty(property)) {
                    this._events[property].dispose();
                }
            }
        }
        enable() {
            this.enabled = true;
        }
        disable() {
            this.enabled = false;
        }
        init() {
            this.scrollObject[this._o.direction] = this._o.from;
            this.maxOffset = this.maskLimits[this._o.direction] * this._o.speedLimit;
            this.enable();
        }
        reset() {
            this.tweenScroll.pause();
            this._o.multiplier = 1;
            this.init();
        }
        setFromTo(from, to) {
            this._o.from = from;
            this._o.to = to;
            this._updateMinMax();
        }
        isTweening() {
            return this.tweenScroll.isRunning;
        }
        registerClickables(clickables) {
            this.clickables = clickables;
        }
        handleDown(target, pointer) {
            if (!this.enabled)
                return;
            this.isDown = true;
            this.target = this.requested = this.scrollObject[this._o.direction];
            this._o.time.down = pointer.timeDown;
            if (this._o.addListeners)
                this.game.input.addMoveCallback(this.handleMove, this);
            if (this.isTweening() && this._o.time.down - this._o.time.up < this._o.accelerationT) {
                this._o.multiplier += this._o.acceleration;
            }
            else {
                this._o.multiplier = 1;
            }
            this.tweenScroll.stop();
            this.tweenScroll.pendingDelete = false;
            util_1.dispatchClicks(pointer, this.clickables, 'onInputDown');
            this._events.onInputDown.dispatch(target, pointer);
        }
        handleMove(pointer, x, y) {
            if (!this.enabled)
                return;
            this.isScrolling = true;
            _ptHelper.set(x, y);
            this.diff = this._old - _ptHelper[this._o.direction];
            this.diff = this._requestDiff(this.diff, this.target, this.min, this.max, this._o.overflow);
            this.target -= this.diff;
            this._old = _ptHelper[this._o.direction];
            this._o.time.move = this.game.time.time;
            this.acc = Math.min(Math.abs(this.diff / 30), this._o.maxAcceleration);
            this.scrollObject[this._o.direction] = this.target;
            this.handleUpdate();
            if (this._o.emitMoving)
                this._events.onInputMove.dispatch(pointer, x, y);
        }
        handleUp(target, pointer) {
            this.isDown = false;
            if (this._o.addListeners)
                this.game.input.deleteMoveCallback(this.handleMove, this);
            this._o.time.up = pointer.timeUp;
            if (this._o.time.up - this._o.time.down > this._o.accelerationT) {
                this._o.multiplier = 1;
            }
            const o = { duration: 1, target: this.target };
            if (!this._o.bouncing)
                o.duration = 0.01;
            if (!this._o.infinite && this.scrollObject[this._o.direction] > this.max) {
                this.target = this.max;
                this.tweenTo(o.duration, this.target);
            }
            else if (!this._o.infinite && this.scrollObject[this._o.direction] < this.min) {
                this.target = this.min;
                this.tweenTo(o.duration, this.target);
            }
            else {
                this._addMomentum(o);
                this._addSwiping(o, pointer);
                this._addSnapping(o);
                this._addLimits(o);
                this._calculateDuration(o);
                this.tweenTo(o.duration, o.target);
            }
            util_1.dispatchClicks(pointer, this.clickables, 'onInputUp');
            this._events.onInputUp.dispatch(target, pointer, util_1.dispatchClicks);
        }
        _addMomentum(o) {
            if (!this._o.momentum)
                return o.target;
            let offset = Math.pow(this.acc, 2) * this.maskLimits[this._o.direction];
            offset = Math.min(this.maxOffset, offset);
            offset = this.diff > 0 ? -this._o.multiplier * offset : this._o.multiplier * offset;
            if (this._o.time.up - this._o.time.move < this._o.flickTimeThreshold && offset !== 0 && Math.abs(offset) > this._o.offsetThreshold) {
                o.target += offset;
            }
            return o;
        }
        _addSwiping(o, pointer) {
            let swipeDistance = Math.abs(this._down - this._current);
            if (this._o.swipeEnabled && this._o.time.up - this._o.time.down < this._o.swipeTimeThreshold && swipeDistance > this._o.swipeThreshold) {
                let direction = pointer[this._o.direction] < this._down ? 'forward' : 'backward';
                if (direction == 'forward') {
                    o.target -= this._o.snapStep / 2;
                }
                else {
                    o.target += this._o.snapStep / 2;
                }
                this._events.onSwipe.dispatch(direction);
            }
            return o;
        }
        static nearestMultiple(n, multiple) {
            return Math.round(n / multiple) * multiple;
        }
        _addSnapping(o) {
            if (!this._o.snapping) {
                return o;
            }
            o.target = Scroller.nearestMultiple(o.target, this._o.snapStep);
            return o;
        }
        _addLimits(o) {
            if (this._o.infinite)
                return o;
            o.target = Math.max(o.target, this.min);
            o.target = Math.min(o.target, this.max);
            return o;
        }
        _calculateDuration(o) {
            let distance = Math.abs(o.target - this.scrollObject[this._o.direction]);
            o.duration = this._o.duration * distance / this.maxOffset;
            o.duration = Math.max(this._o.minDuration, o.duration);
            return o;
        }
        _requestDiff(diff, target, min, max, overflow) {
            if (this._o.infinite)
                return diff;
            let scale = 0;
            if (target > max) {
                scale = (max + overflow - target) / overflow;
                diff *= scale;
            }
            else if (target < min) {
                scale = -(min - overflow - target) / overflow;
                diff *= scale;
            }
            return diff;
        }
        tweenToSnap(duration, snapIndex) {
            let target = this._o.from - this._o.snapStep * snapIndex;
            this.tweenTo(duration, target);
        }
        tweenTo(duration, target) {
            if (duration == 0)
                return this.setTo(target);
            const o = { [this._o.direction]: target };
            this.tweenScroll.onUpdateCallback(this.handleUpdate, this);
            this.tweenScroll.onComplete.add(this.handleComplete, this);
            this.tweenScroll.updateTweenData('vEnd', o, -1);
            this.tweenScroll.updateTweenData('duration', duration * 1000, -1);
            this.tweenScroll.updateTweenData('percent ', 0, -1);
            this.tweenScroll.start();
        }
        cancel() {
            this.isDown = false;
        }
        setTo(target) {
            this.scrollObject[this._o.direction] = target;
            this.tweenScroll.stop();
            this.handleUpdate();
            this.handleComplete();
        }
        static percentageBetween2(input, lo, hi) {
            return (input - lo) / (hi - lo);
        }
        handleUpdate() {
            if (!this.enabled)
                return;
            if (this._o.infinite) {
                this.dispatchValues.total = Phaser.Math.wrap(this.scrollObject[this._o.direction], this.min, this.max);
            }
            else {
                this.dispatchValues.total = this.scrollObject[this._o.direction];
            }
            let step = this.dispatchValues.total - this.previousTotal;
            if (step < -this.length / 2) {
                step = step + this.length;
            }
            else if (step > this.length / 2) {
                step = step - this.length;
            }
            this.dispatchValues.step = step;
            this.dispatchValues.percent = Scroller.percentageBetween2(this.dispatchValues.total, this._o.from, this._o.to);
            this._events.onUpdate.dispatch(this.dispatchValues);
            this.previousTotal = this.dispatchValues.total;
        }
        handleComplete() {
            if (!this.enabled)
                return;
            this.isScrolling = false;
            this._o.multiplier = 1;
            this._events.onComplete.dispatch();
        }
        _updateMinMax() {
            this.min = Math.min(this._o.from, this._o.to);
            this.max = Math.max(this._o.from, this._o.to);
            this.length = Math.abs(this.max - this.min);
            this.previousTotal = this._o.from;
        }
    }
    exports.Scroller = Scroller;
});
//# sourceMappingURL=scroller.js.map